/*!
* \file controleur.c
* \brief Fichier contenant le code des fonctions du controleur.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "partie.h"
#include "joueur.h"
#include "joueurList.h"
#include "case.h"
#include "personnage.h"
#include "classe.h"
#include "gamePackage.h"
#include "roxml.h"
#include "SDL.h"
#include "structures.h"

Partie * partieGlobale;
Game_Package packGlobal;

  /*!
 * \fn void initJeu()
 * \brief La fonction initialise les variable globales du package du jeu et de la Partie
 */
void initJeu(){
    partieGlobale = initPartie();
    packGlobal = Construire_Package();
}

/*!
 * \fn void ajouterUnJoueur(char * nomDuJoueur, EtatsJeu * etat)
 * \brief Fonction ajoutant un Joueur  la Partie et faisant passer la partie  l'tat suivant.
 *        Si le joueur n'a pas correctement t ajout, l'tat reste le mme.
 *
 * \param Le nom du joueur  ajouter  la partie
 */
 void ajouterUnJoueur(char * nomDuJoueur, EtatsJeu * etat){
    Joueur * j = nouveauJoueur();
    if(j!=NULL){
        setNomJoueur(j,nomDuJoueur);
        if(addParticipant(partieGlobale,j)==1){
            *etat =LANCEMENT;
        }
        else{
            *etat=SAISIE_JOUEURS;
        }
    }
    else{
        *etat=SAISIE_JOUEURS;
    }
 }

  /*!
 * \fn  int nbClasses()
 * \brief Fonction renvoyer le nombre de classes contenues dans le package du jeu
 *
 * \return le nombre de classes contenues dans le package du jeu
 */
 int nbClasses(){
    return getNbClasses(packGlobal);
}

 /*!
 * \fn char ** obtenirNomsClasses()
 * \brief Fonction renvoyant un tableau de chaine de caractres contenant les noms des Classes.
 *        L'ordre et l'index des noms dans le tableau correspond  l'ordre dans lequel ils sont dfinis
 *        dans le fichier XML.
 *
 * \return un tableau de chaine de charactres (char**).
 */
 char ** obtenirNomsClasses(){
    int i;
    //Allocation du tableau contenant les chaines de charactres
    char ** tabNomClasses=malloc(nbClasses()*sizeof(char[TAILLE_NOMS]));
    for(i=0;i<nbClasses();i++){
        tabNomClasses[i]= malloc(TAILLE_NOMS*sizeof(char));
    }
    //Copie des noms de classes dans le tableau
    for(i=0;i<nbClasses();i++){
        strncpy(tabNomClasses[i],packGlobal.Classes_Package[i].nom, TAILLE_NOMS);
    }
    return tabNomClasses;
 }

  /*!
 * \fn char ** obtenirSkinsClasses()
 * \brief Fonction renvoyant un tableau de chaine de caractres contenant le chemin vers les Skins des Classes.
 *        L'ordre et l'index des noms dans le tableau correspond  l'ordre dans lequel ils sont dfinis
 *        dans le fichier XML.
 *        Ici le chemin du Skin renvoy est celui du "default".
 *
 * \return un tableau de chaine de charactres (char**).
 */
 char ** obtenirSkinsClasses(){
    int i;
    //Allocation du tableau contenant les chaines de charactres
    char ** tabSkinsClasses = malloc((getNbClasses(packGlobal))*sizeof(char *));
    for(i=0;i<getNbClasses(packGlobal);i++){
        tabSkinsClasses[i]= malloc(TAILLE_CHEMIN*sizeof(char));
    }
        //Copie des chemin des skins dans le tableau
    for(i=0;i<getNbClasses(packGlobal);i++){
        strncpy(tabSkinsClasses[i],packGlobal.Classes_Package[i].skins.charac_default, TAILLE_CHEMIN);
    }
    return tabSkinsClasses;
 }

   /*!
 * \fn void freeClasses()
 * \brief La fonction libre un tableau de chaines de charactres allou pour stocker les informations sur les classes.
 *
 * \param
 */
  void freeClasses(char ** tab){
    int i;
    for(i=0;i<nbClasses();i++){
        free(tab[i]);
    }
    free(tab);
}

   /*!
 * \fn  int nbCartes()
 * \brief Fonction renvoyer le nombre de cartes contenues dans le package du jeu
 *
 * \return le nombre de cartes contenues dans le package du jeu
 */
int nbCartes(){
    return getNbCartes(packGlobal);
}

  /*!
 * \fn char ** obtenirNomsCartes()
 * \brief Fonction renvoyant un tableau de chaine de caractres contenant les noms des Cartes.
 *        L'ordre et l'index des noms dans le tableau correspond  l'ordre dans lequel ils sont dfinis
 *        dans le fichier XML.
 *
 * \return un tableau de chaine de charactres (char**).
 */
char ** obtenirNomsCartes(){
    int i;
    //Allocation du tableau contenant les chaines de charactres
    char ** tabNomsCartes = malloc((getNbCartes(packGlobal))*sizeof(char *));
    for(i=0;i<getNbCartes(packGlobal);i++){
        tabNomsCartes[i]= malloc(TAILLE_CHEMIN*sizeof(char));
    }
    //Copie des noms des cartes dans le tableau
    for(i=0;i<getNbCartes(packGlobal);i++){
        strncpy(tabNomsCartes[i],packGlobal.Cartes_Package->nom, TAILLE_CHEMIN);
    }
    return tabNomsCartes;
}

  /*!
 * \fn char ** obtenirDescriptionCarte()
 * \brief Fonction renvoyant un tableau de chaine de caractres contenant les descriptions des Cartes.
 *        L'ordre et l'index des descriptions dans le tableau correspond  l'ordre dans lequel ils sont dfinis
 *        dans le fichier XML.
 *
 * \return un tableau de chaine de charactres (char**).
 */
char ** obtenirDescriptionCarte(){
    int i;
    //Allocation du tableau contenant les chaines de charactres
    char ** tabDescCartes = malloc((getNbCartes(packGlobal))*sizeof(char *));
    for(i=0;i<getNbCartes(packGlobal);i++){
        tabDescCartes[i]= malloc(100*sizeof(char));
    }
    //Copie des descriptions des cartes dans le tableau
    for(i=0;i<getNbCartes(packGlobal);i++){
        strncpy(tabDescCartes[i],packGlobal.Cartes_Package->desc, 100);
    }
    return tabDescCartes;
}

  /*!
 * \fn void freeCartes()
 * \brief La fonction libre un tableau de chaines de charactres allou pour stocker les informations sur la carte.
 *
 * \param le tableau de charactres  supprimmer.
 */
  void freeCartes(char ** tab){
    int i;
    for(i=0;i<nbCartes();i++){
        free(tab[i]);
    }
    free(tab);
}

/*!
 * \fn Case * trouverCase(Carte c, int x, int y)
 * \brief La fonction retourne un pointeur vers la case de coordonnes x,y
 *
 * \param les coordonnes x et y d'une case (pas en pixels mais bien en colonne et ligne), ainsi que la carte utilise
 * \return un pointeur vers cette case
 */
Case * trouverCase(Carte_Jeu * c, int x, int y){
    if((x>=0)&&(x<LARG_MAX_CARTE)&&(y>=0)&&(y<HAUT_MAX_CARTE)){
       return &c->Tab_Cases[x][y];
    }
    return NULL;
}

  /*!
 * \fn int ajouterPersonnage(char * nomClasse, int numTab, int numJoueur)
 * \brief La fonction ajoute un personnage au tableau de personnages d'un Joueur  l'indice numTab(compris entre 0 et TAILLE_MAX_GROUPE).
 *        S'il y a dj un personnage  l'indice numTab, l'ancien personnage est effac du tableau.
 *
 * \param le nom de la classe dont dpend le Personnage, numTab un entier reprsentant l'indice auquel ajouter le Personnage dans le table de personnages,
 *        numJoueur est un entier reprsentant le premier ou le second Joueur de la Partie.
 * \return L'indice de la classe du personnage si le Personnage a t correctement ajout, -1 sinon.
 */
int ajouterPersonnage(char * nomClasse, int numTab, int numJoueur){
    int i;
    i=0;
    //Recherche de la classe dans le Package
    while(strcmp(nomClasse,getNomClasse(&packGlobal.Classes_Package[i]))!=0&&i<nbClasses()){
        i++;
    }
    if(i>=nbClasses()){
        return -1;
    }

    //Cration du personnage
    Personnage * p = default_personnage(i,packGlobal);

    printf("La classe du personnage %d du joueur %d est %s\n", numTab+1, numJoueur, getClasse(p));

    //Ajout du Personnage  l'quipe du Joueur
    switch (numJoueur){
    case 1:
        setOnFirstJoueur(partieGlobale->participants);
        break;
    case 2:
        setOnFirstJoueur(partieGlobale->participants);
        nextJoueur(partieGlobale->participants);
        break;
    default:
        return -1;
    }

    if((numTab<TAILLE_MAX_GROUPE)&&(numTab>=0)){
        setNumero(p,numTab);
        addPersonnage(getCurrentJoueur(partieGlobale->participants),p,numTab);
        return i;
    }
    else{
        return -1;
    }
}

  /*!
 * \fn int selectionnerCarte(char * nomCarte)
 * \brief La fonction ajoute la Carte dont le nom est en parrametre  la Partie.
 *        Les Terrains de la Carte sont initialis  partir des cartes dcrites en XML
 *        Les Personnages sont placs  leur position de dpart sur la carte.
 *
 * \param Le nom de la Carte dcrite en XML.
 * \return 1 si la carte est correctement initialise, 0 sinon.
 */
int selectionnerCarte(char * nomCarte){
    int i;
    i=0;
    //Recherche de la Carte dans le Package
    while((strcmp(nomCarte,packGlobal.Cartes_Package[i].nom)!=0)&&(i<nbCartes())){
        i++;
    }
    if(i>=nbCartes()){
        return 0;
    }
    //Initialisation de la Carte_Jeu
    if(initCarte(&packGlobal.Cartes_Package[i],getCarte(partieGlobale),&packGlobal)==0){
        return 0;
    }

    // Positionnement des Personnages sur la Carte_Jeu.
    setOnFirstJoueur(partieGlobale->participants);
    for(i=0;i<TAILLE_MAX_GROUPE;i++){
        setPosition(getCurrentJoueur(partieGlobale->participants)->groupe[i],&partieGlobale->c->Tab_Cases[6+i][0]);
    }
    nextJoueur(partieGlobale->participants);
    for(i=0;i<TAILLE_MAX_GROUPE;i++){
        setPosition(getCurrentJoueur(partieGlobale->participants)->groupe[i],&partieGlobale->c->Tab_Cases[6+i][HAUT_MAX_CARTE-1]);
    }
    return 1;
}




